# -*- coding: utf-8 -*-
"""
Created on Wed Nov 6 10:10:04 2018
@author: fc_ca
"""
from PIL import Image, ImageDraw, ImageFont
import os, time
import drawLines, shutil, supportGeneration, debugOptions #local files

from xml.etree.ElementTree import ElementTree
from xml.etree.ElementTree import Element
import xml.etree.ElementTree as etree
    
def mainLoop(previewOn,cubeHeight,xDim,yDim,layerheight,memberSize,cubesXdir,cubesYdir,spacingUM,RGBval,RGBcorner,zipfileName):
    
    start = time.time() #start stop watch
 
    #----User defined input----------------------------------------------------
    cubeZheiht = cubeHeight
    cubeXsize = int(xDim*1000/50) #convert size in pixels, where on pixel = 50uM
    cubeYsize = int(yDim*1000/50) #convert size in pixels, where on pixel = 50uM
    spacing = int(((spacingUM)+(memberSize))/50)
  
    
    #------Initial calculations----------------------------------------------------
    totalLayers = int(cubeZheiht*1000//layerheight) #Converts mm to a round number of layers based on layerheight
    solidLayer = int(memberSize/layerheight)
    postLayer =  2*spacing*(-1) # amount of layers with only vertical features (posts). Must be changed to account for various layerheights.
    namespace = str(totalLayers) #used for generating leading zeroes in picture name
    cubeSize = max(cubeXsize,cubeYsize)# cubeXYsize*1000//50 
    
    
    #--------Support options----------------------------------
    supportLayers = (((postLayer*(-1)+solidLayer))%40+(postLayer*(-1)+solidLayer))*2#support layers number
    platformOnlyLayers = supportLayers
    platformType = "frame" #choose solid or square (frame)
    frameThickness = 1.6 #mm
    
    
    #---------Info print at program strart-------------------------------------
    print("\nParameters in use:\n")
    print("Part dimensions (Xmm * Ymm) = "+str(xDim)+" * "+str(yDim)+" mm, or "+str(cubeXsize)+(" x ") + str(cubeYsize)+" pixels")
    print("Layer thickness = " +str(layerheight)+"uM \nTotal number of layers = " + str(totalLayers)+", which equals a total height of "+str(totalLayers*layerheight/1000)+"mm")
    print("of which support is "+str(platformOnlyLayers*layerheight/1000)+"mm")
    print("\nMember size  = " +str(memberSize)+"uM")
    print("Gaps between beams horizontally = "+str(spacing*50-memberSize)+"uM")
    print("Number of layers horizontal grids = " + str(solidLayer))
    print("Number of layers with only vertical posts = " +str(postLayer*-1))
    print("Supportlayers " + str(supportLayers))
    
    #--------Font options for labels on parts----------------------------------
    fontsize = 25 #40 = 1mm tall letters
    fntSmall=ImageFont.truetype("arial.ttf",fontsize)
    fntLarge = ImageFont.truetype("impact.ttf",fontsize*4)
    currentFont = fntSmall
    #Output request
    saveImageFiles = True #True = on, False = off |
    showImageFiles = previewOn #True = on, False = off |
    
    #---------Initial coordinates of cubes on buildplatform--------------------
    origXstart = 80 #Starting point in x-direction
    origYstart = 80 # starting point in y-direction
    xspacing = 120 #horizontal spacing between cubes
    yspacing = 110 #vertical spacing between cubes
    
    #----------XML-file-structure----------------------------------------------
    root=Element("root")
    tree=ElementTree(root)

    name=Element('name')
    root.append(name)
    name.text='neutral'
    
    data0=Element('data0')
    root.append(data0)
    data0.text='1'
    
    data1=Element('data1')
    root.append(data1)
    data1.text='1'
    
    data2=Element('data2')
    root.append(data2)
    data2.text='0'

    #------------Main-drawing-loop-------------------------------------------------
    sLayer = solidLayer #used for counting horizontal mesh layers and between layers with just vertical beams

    print("Entering layer generation loop...\n") # Nested for/while loops to draw mesh
    
    for layer in range (0, (totalLayers)):
      
        img = Image.new('RGB', (2560,2400), color = 'black')
        d = ImageDraw.Draw(img)
        
        xStart = origXstart; #Initialize position variables
        yStart = origYstart; #Initialize position variables
        
        xEnd = xStart + cubeXsize #Initialize position variables
        yEnd = yStart + cubeYsize #Initialize position variables
        
        debugOptions.drawZones(d,layer,currentFont) #Activate to show print zones and layer number
        
        for y in range(0, cubesYdir): #Vertical offsetting loop
             
            for x in range(0,cubesXdir): #draw x number of cubes horisontaly 
                            
                if layer>=platformOnlyLayers: #draw structures
                    supportGeneration.drawSupport(d,layer,supportLayers,platformType,frameThickness,xStart,yStart,xEnd,yEnd,RGBval,RGBcorner,memberSize,spacing,currentFont,cubeSize,sLayer,platformOnlyLayers)
                    drawLines.whiteLines(d,cubeSize, memberSize,RGBval,spacing,xStart, yStart, xEnd, yEnd,sLayer,0,platformType,RGBcorner,layer,frameThickness)
                    drawLines.whiteLines(d,cubeSize, 300,255,int(3000/50),xStart, yStart, xEnd, yEnd,sLayer,1,platformType,RGBcorner,layer,frameThickness)
                
                else: #draw only support structure
                    supportGeneration.drawSupport(d,layer,supportLayers,platformType,frameThickness,xStart,yStart,xEnd,yEnd,RGBval,RGBcorner,memberSize,spacing,currentFont,cubeSize,sLayer,platformOnlyLayers)
                
                #---next---cube-in Xdir-----------------
                xStart = xStart + cubeXsize+xspacing #traverse horisontaly
                xEnd = xEnd + cubeXsize+xspacing #
                
                if (1280 - (xStart%1280))< cubeSize+xspacing: #a check if object overlaps zone
                    xStart = 1280+origXstart #if yes, move to next zone
                    xEnd = xStart + cubeXsize
                   
                # RGBval -= 5; #Activaty to gradually reduce RGB intensity pr cube
                #RGBcorner -=5 #Activaty to gradually reduce RGB intensity pr cube
                #-----------------y-movement----------------------------------------------
         
            yStart = yStart + cubeSize + yspacing #traverse build platform vertically
            yEnd  = yEnd + cubeSize + yspacing # same
            
            if      (800-(yStart%800)) < cubeSize+yspacing: # check if near a cell edge (every 800 pixel)
                    
                    if(yStart < 800): #if colission in first sone, move to zone 2 (starts at 800 pixel)
                        yStart = 800 + origYstart
                        yEnd  = yStart +  cubeSize
                        
                    else: #if we colission in zone 2, move to zone 3 (starts at 1600 pixel)
                        yStart = 1680
                        yEnd  = yStart +  cubeSize       
            
            xStart = origXstart #reset x-position after a vertical iteration
            xEnd = origXstart+cubeXsize
        
        #RGBval = 200;#Activaty to reset RGB after each layer
        #RGBcorner = 200; #Activaty to reset RGB after each layer
           
            #-------------count solid vs post layers-------------------------------
        if (sLayer-1) == postLayer:#(-spacing*solidLayer)+(solidLayer+1):   #print posts from 0,-1,-2,-3,-4,-5 = 150uM spacing
            #print("restet solidlayer = {}, slayer{}".format(solidLayer,sLayer))
    
            sLayer =solidLayer #* multiGrid; #reset solid layer counter
        else: #continue countdown of solid layers
            sLayer -=1
            
        #--------------------show & save current png-------------------------------
        number = str(layer).zfill(len(namespace)) #generate number to add to name
        #r = range(supportLayers,supportLayers+solidLayer)
        if showImageFiles == True and (supportLayers+solidLayer-1) <= layer <= supportLayers+solidLayer:
            img.show() #show first normal layer image
        if saveImageFiles == True:
            curdir= str(os.getcwd())
            img.save(curdir+'\imgout\layer_{}.png'.format(number),compress_level=1) #save layer as image in png format with {number} added

        #------------------------------XML-data-appended-pr.layer------------------------

        zv = round((layer+1)*layerheight/1000,3) #store current z-value (layerheight) in mm rounded to 3 digits
        zvs = format(zv, '.3f') #store zvalue as a string with 3 digits behind comma allways: 0.100, 0.250 etc

        xml_layer=Element('layer')
        root.append(xml_layer)
        
        index=Element('index')
        xml_layer.append(index)
        index.text=str(layer)
        
        zvalue=Element('zvalue')
        xml_layer.append(zvalue)
        zvalue.text=str(zvs)
        
        filename=Element('filename')
        xml_layer.append(filename)
        filename.text='layer_{}.png'.format(number)
    
        #print(etree.tostring(root)) #printing tree structure to console
        tree.write(curdir+"\imgout\index.xml")
        
        if totalLayers - layer < ((postLayer*-1)+solidLayer) and sLayer == 0: #ensure top side of grid is closedd
          print("Stopped to ensure solid top layers (no open beams)")
          break;
        #---------------------------END OF LAYER FOR LOOP--------------------------

    shutil.make_archive(zipfileName, 'zip', curdir+'\imgout')
    
    end = time.time()
    print("Finished generating {} layers to ".format(layer+1) +"\nTotal height = " + str((layer+1)*layerheight/1000)+"mm") #end of program
    print("Total time = " +str(end-start)+" seconds")
    print("Saved to "+curdir+"\imgout\\"+ zipfileName+".zip \n") #end of program